﻿/*utf8解析行 Don't remove or modify this first line!*/
#ifndef WNWORKTHREAD_H
#define WNWORKTHREAD_H

//#include "WNInvoker.h"
#include "WNCreticalSection.h"
#include "WNNotificationCenter.h"
#include "WNCLoopList.h"
#include <pthread.h>

class WNWorkThread;
class WNNotification;
class WNNotificationObserver;
class WNNotificationCenter;

#define WN_WORK_THREAD_EXIT false
#define WN_WORK_THREAD_CONTINUE true

typedef pthread_t WNThreadId;

enum ThreadResetFlag
{
	TRF_ResetMember, //重置成员信息
	TRF_DeleteInstance //delete实例
};

class WNWorkThreadDelegate
{
public:
    virtual bool OnWorkThreadStart(WNWorkThread* objThread) { return WN_WORK_THREAD_CONTINUE; }
    virtual bool OnWorkThreadProc(WNWorkThread* objThread, unsigned int& uStep) { return WN_WORK_THREAD_EXIT; }
    virtual unsigned int OnWorkThreadProcEnd(WNWorkThread* objThread) { return 0; }
    virtual unsigned int OnWorkThreadNeedSleep(WNWorkThread* objThread) { return 1; }
    virtual ~WNWorkThreadDelegate(){}
};


class WNThreadObserverInfoItem
{
public:
    WNThreadObserverInfoItem();
    WNNotificationObserver* mpObserver;
    void* mpSender;
};

typedef WNCLoopList<WNThreadObserverInfoItem> WNThreadObserverInfoItemList;

class WNThreadObserverInfo
{
public:
    WNThreadObserverInfo();
    unsigned int muNotificationId;
    WNNotificationCenter* mpCenter;
    WNThreadObserverInfoItemList mObserverList;
};

typedef WNCLoopList<WNThreadObserverInfo> WNThreadObserverInfoList;

class WNWorkThread : public WNNotificationObserver//, public WNInvoker
{
public:
    typedef void (*WNThreadDestroyFn)(WNWorkThread* pThread);

public:
	WNWorkThread(void);
	virtual ~WNWorkThread(void);

    static unsigned int GetTickCount();
    //线程睡眠，默认单位是毫秒
    //最终的睡眠时间是uTimeOut * uMultiplier，单位是微妙
    static void Sleep(unsigned int uTimeout, unsigned uMultiplier = 1000);

    static void DoEvents(WNWorkThread* pThread = NULL);

    static void DoEvents(unsigned int uTimeOut, WNWorkThread* pThread = NULL);

    static void DoEventsAndSleep(unsigned int uSleepTime, WNWorkThread* pThread = NULL);

    static void DoEventsAndSleep(unsigned int uTimeOut, unsigned int uSleepTime, WNWorkThread* pThread = NULL);

	static WNWorkThread* GetCurrentThread();
	static WNWorkThread* GetMainThread();

    //外部线程调用，结束指定线程
    //linux下无法使用
    static bool DoTerminateThread(WNThreadId threadId, unsigned int uExitCode = 0 );
	//内部线程调用，结束当前函数执行线程
    //不建议直接调用，建议使用EndWork()结束线程
    static void DoExitThread(unsigned int uExitCode = 0);
	virtual void StartWork();
	//结束线程
	//dwTimeOut表示正常结束后超出多长时间后强制结束线程
    //linux下强制结束线程尚未实现
    virtual void EndWork(unsigned int dwTimeOut = 500);

    //返回是否为主线程
    virtual bool IsMainThread();
    //返回线程是否正在运行
    virtual bool IsThreadRunning();
    //返回是否为当前线程
    virtual bool IsCurrentThread();


	inline WNWorkThreadDelegate*& Delegate() { return mpThreadDelegate; }
    inline WNThreadId ThreadId() { return mThreadId; }
	inline ThreadResetFlag& ResetFlag( ) { return mResetFlag; }
    inline WNThreadDestroyFn ThreadDestroyFn() { return mpThreadDestroyFn; }

    //在当前线程添加观察者，指定发送者以及消息中心的通知(字符串)，当前线程作为消息中转站
    //只有当线程在工作的时候才会有效
    //pObserver是观察者
    //lpNotificationName是要观察的通知名称
    //pSender是发送者，为NULL时表示观察所有发送者发送的lpNotificationName
    //pNotification是指定的消息中心,为默认值NULL是表示使用WNNotificationCenter::DefaultCenter()
    //返回值为通知Id，返回值为非0表示成功，否则表示失败
    virtual unsigned int AddObserve4Notification(WNNotificationObserver* pObserver, const char* lpNotificationName, void* pSender = NULL, WNNotificationCenter* pNotificationCenter = NULL);

    //在当前线程移除观察者，指定发送者以及消息中心的通知(字符串)，当前线程作为消息中转站
    //只有当线程在工作的时候才会有效
    //pObserver是观察者
    //lpNotificationName是要停止观察的消息名称
    //pSender是发送者，为NULL时表示停止观察所有发送者发送的lpNotificationName所对应的消息
    //当lpNotificationName为NULL时表示停止观察所有的消息
    //pNotification是指定的消息中心,为默认值NULL是表示使用WNNotificationCenter::DefaultCenter()
    virtual void RemoveObserve4Notification(WNNotificationObserver* pObserver, const char* lpNotificationName = NULL, void* pSender = NULL, WNNotificationCenter* pNotificationCenter = NULL);

    //向当前线程投递消息
    virtual bool PostThreadMessage(WNNotification *pNotification);

    //处理队列中的线程消息
    //返回false表示线程队列中无消息
    //返回true表示处理了一个线程消息
    virtual bool ProcessMessageInQueue();
    //清空线程消息队列
    virtual void ClearMessageQueue();
protected:
	virtual void OnReceiveNotification(WNNotification* pNotification);
protected:
    static void* InternalThreadMain( void* lpParam );
    //创建线程
    //成功返回true，失败返回false
    virtual bool CreateThread();
    virtual unsigned int ThreadMain();
    virtual bool ThreadProc(unsigned int uStep);
	//重置为初始状态(StartWork之前的状态)
	//mpThreadDelegate不会重置
	virtual void DoReset();
    //等待线程结束，在外部线程使用
	//dwTimeOut表示超时
	//如果在dwTimeOut之内正常结束，则返回true，否则返回false
    virtual bool WaitThreadEnd(unsigned int dwTimeOut = 500);

protected:
	WNWorkThreadDelegate* mpThreadDelegate;
    volatile bool mbExitOutThread; //线程外退出标志
    volatile bool mbExitInThread; //线程内退出标志
    WNThreadId mThreadId;
    void* mThreadHandle;
    unsigned int muLastThreadExitCode; //最后一次线程退出代码
    volatile bool mbIsThreadRunning; //线程是否正在运行
    ThreadResetFlag mResetFlag; //线程重置标志位，线程退出时有效，TRF_DeleteInstance只能在线程自己结束的情况下使用
    WNThreadDestroyFn mpThreadDestroyFn; //线程销毁函数
    WNCLoopList<WNNotification*> mMsgQueue; //消息处理队列
    WNCreticalSection mMsgQueueLocker; //消息处理队列锁
    WNThreadObserverInfoList mObserverInfoList; //消息转发映射表
    WNCreticalSection mObserverListLocker; //消息转发映射表锁
};

#endif
