/*utf8解析行 Don't remove or modify this first line!*/
#ifdef __WINOS__
#include "stdafx.h"
#endif
#include "WNBitIndex.h"
#include "WNObject.h"
#include <memory.h>

#define LeafsInfoCount 65536
#define ValueCount 16

//保存WNBitNode(WNBitNodeEx、WNByteNode)类中muLeafsInfo与mppLeafs的关系
//如要取得mppLeafs中数值为v(范围0-15)的节点的方法如下：
//    1、检查v所对应的节点序号是否存在，即判定mLeafsInfoMap[muLeafsInfo][v]是否为0
//    2、如v所对应的节点序号index不为0，则通过mppLeafs[index - 1]即可获得节点指针
//条件语句如果下：
//    return mLeafsInfoMap[muLeafsInfo][v]?mppLeafs[mLeafsInfoMap[muLeafsInfo][v] - 1]:NULL;
unsigned int mLeafsInfoMap[LeafsInfoCount][ValueCount+1] = { { 0 } };


//节点值所对应的muLeafsInfo中的位
unsigned int mLeafsValueBit[ValueCount] = { 
									0x1, 0x2, 0x4, 0x8, 
									0x10, 0x20, 0x40, 0x80, 
									0x100, 0x200, 0x400, 0x800, 
									0x1000, 0x2000, 0x4000, 0x8000
								  };

//默认的子节点列表，全0，包含Data
WNBitNode* mDefLeafs[ValueCount+1] = { 0 };

#define LowHalfByte 0
#define HighHalfByte 1
//保存一个字节的高4位和低4位值的数组
unsigned int mHalfByteValue[256][2] = { { 0 } };

//用于创建WNBitNode的叶子信息(muLeafsInfo)与叶子节点(mpLeafs)的对照表
class WNBitNodeLeafsInfoMapCreator
{
public:
	WNBitNodeLeafsInfoMapCreator();
};

//用于初始化mLeafsInfoMap
WNBitNodeLeafsInfoMapCreator mBitNodeLeafsInfoMapCreator;

//==================================================================================================
//WNBitNodeLeafsInfoMapCreator======================================================================
//==================================================================================================
WNBitNodeLeafsInfoMapCreator::WNBitNodeLeafsInfoMapCreator()
{
	//初始化节点信息与节点的映射表
	for (int i = 0; i < LeafsInfoCount; i++)
	{
		int iIndex = -1;
		int iValue = i;
		int iValueCount = ValueCount;
		int iNextLocal = 1;
		//子节点数量置0
		mLeafsInfoMap[i][ValueCount] = 0;
		while (iValueCount--)
		{
			iIndex++;
			if ((iValue & 1))
			{
				//有值，设置节点位置，从1开始
				mLeafsInfoMap[i][iIndex] = iNextLocal;
				iNextLocal++;
				//增加子节点数量
				mLeafsInfoMap[i][ValueCount] += 1;
			}
			else
			{
				mLeafsInfoMap[i][iIndex] = 0;
			}
			iValue = (iValue >> 1);
		}
	}
	//初始化字节高4位以及低4位值信息
	for (int i = 0; i < 256; i++)
	{
		mHalfByteValue[i][LowHalfByte] = i & 0x0F;
		mHalfByteValue[i][HighHalfByte] = (i >> 4) ;
	}
}



//===================================================================================================
//WNBitNode==========================================================================================
//===================================================================================================

#define ValueIndex (mLeafsInfoMap[muLeafsInfo][ValueCount])
#define ArraySizeWithValue (mLeafsInfoMap[muLeafsInfo][ValueCount] + 1)
#define ArraySizeWithoutValue (mLeafsInfoMap[muLeafsInfo][ValueCount])

WNBitNode* WNBitNode::NullPtr = 0;

WNBitNode::WNBitNode()
{
	memset(this,0x00, sizeof(WNBitNode));
	mppLeafs = mDefLeafs;
}


WNBitNode::~WNBitNode()
{
	if (mppLeafs != mDefLeafs)
	{
		//有叶子节点或者数据
		if (mbHasData && mbIsWNObjectPtr)
		{
			//有数据
			if (mbIsWNObjectPtr)
			{
				((WNObject*)(Data()))->Release();
			}
		}
		if (!mbDoNotFreeLeafs)
		{
			//删除所有子节点
			unsigned int uCount = LeafsCount();
			for (unsigned int i = 0; i < uCount; i++)
			{
				delete mppLeafs[i];
			}
		}
		//删除子节点以及数据列表
		delete[] mppLeafs;
	}
}

void WNBitNode::ReInit()
{
	if (mppLeafs != mDefLeafs)
	{
		//有叶子节点或者数据
		if (mbHasData && mbIsWNObjectPtr)
		{
			//有数据
			if (mbIsWNObjectPtr)
			{
				((WNObject*)(Data()))->Release();
			}
		}
		if (!mbDoNotFreeLeafs)
		{
			//删除所有子节点
			unsigned int uCount = LeafsCount();
			for (unsigned int i = 0; i < uCount; i++)
			{
				delete mppLeafs[i];
			}
		}
		//删除子节点以及数据列表
		delete[] mppLeafs;
	}
	memset(this,0x00, sizeof(WNBitNode));
	mppLeafs = mDefLeafs;
}

inline WNBitNode* WNBitNode::ChildNode( unsigned int uChildValue )
{
	return mLeafsInfoMap[muLeafsInfo][uChildValue]?mppLeafs[mLeafsInfoMap[muLeafsInfo][uChildValue] - 1]:NULL;
}


inline WNBitNode*& WNBitNode::ChildNodeRef( unsigned int uChildValue )
{
	return mLeafsInfoMap[muLeafsInfo][uChildValue]?mppLeafs[mLeafsInfoMap[muLeafsInfo][uChildValue] - 1]:NullPtr;
}

WNBitNode* WNBitNode::AttatchChildNode( WNBitNode* pNewNode, bool bRemoveOldNode /*= true*/ )
{
	//获得旧节点
	WNBitNode* pOldChild = ChildNode(pNewNode->muNodeValue);
	//判断要添加的节点与旧节点是否相同
	if (pNewNode == pOldChild)
	{
		//要添加的节点与已有节点为同一节点
		return NULL;
	}
	if (!pOldChild)
	{
		//旧节点不存在
		//添加新节点
		//首先得到新的节点信息
		unsigned int uNewLeafsInfo = muLeafsInfo | mLeafsValueBit[pNewNode->muNodeValue];
		//创建新的节点数组
		WNBitNode** pNewLeafs = new WNBitNode*[LeafsAndValueCount() + 1];
		//拷贝原数组的内容至新的节点数组
		unsigned int uNewIndex = 0;
		for (unsigned int i = 0; i < LeafsCount(); i++)
		{
			//得到当前第i个子节点的新序号
			uNewIndex = mLeafsInfoMap[uNewLeafsInfo][mppLeafs[i]->muNodeValue] - 1;
			pNewLeafs[uNewIndex] = mppLeafs[i];
		}
		//拷贝数据
		if (mbHasData)
		{
			pNewLeafs[mLeafsInfoMap[uNewIndex][ValueCount]] = (WNBitNode*)Data();
		}
		//删除元节点数组
		if (mppLeafs != mDefLeafs)
		{
			delete mppLeafs;
			mppLeafs = mDefLeafs;
		}
		//设置新数组
		mppLeafs = pNewLeafs;
		//设置新节点信息
		muLeafsInfo = uNewLeafsInfo;
	}
	//设置新节点
	ChildNodeRef(pNewNode->muNodeValue) = pNewNode;
	//返回旧节点
	return pOldChild;
}


WNBitNode* WNBitNode::DetatchChildNode( unsigned int uChildValue )
{
	//获得要分离的节点
	WNBitNode* pTargetNode = ChildNode(uChildValue);
	if (pTargetNode)
	{
		//存在指定值的节点
		//从节点列表中删除指定的节点
		//首先获得新节点列表
		//首先得到新的节点信息
		unsigned int uNewLeafsInfo = muLeafsInfo ^ mLeafsValueBit[uChildValue];
		//创建新的节点数组
		WNBitNode** pNewLeafs = new WNBitNode*[LeafsAndValueCount() - 1];
		//拷贝原数组内容至新数组(不包含uChildValue节点)
		unsigned int uNewIndex = 0;
		for (unsigned int i = 0; i < LeafsCount(); i++)
		{
			//判断第i个子节点是否为要分离的节点
			if (mppLeafs[i] != pTargetNode)
			{
				//不是要分离的节点，拷贝
				//得到当前第i个子节点的新序号
				uNewIndex = mLeafsInfoMap[uNewLeafsInfo][mppLeafs[i]->muNodeValue] - 1;
				pNewLeafs[uNewIndex] = mppLeafs[i];
			}
		}
		//拷贝数据
		if (mbHasData)
		{
			pNewLeafs[mLeafsInfoMap[uNewIndex][ValueCount]] = (WNBitNode*)Data();
		}
		//删除元节点数组
		if (mppLeafs != mDefLeafs)
		{
			delete mppLeafs;
			mppLeafs = mDefLeafs;
		}
		//设置新数组
		mppLeafs = pNewLeafs;
		//设置新节点信息
		muLeafsInfo = uNewLeafsInfo;
	}
	//返回要分离的节点
	return pTargetNode;
}


WNBitNode* WNBitNode::AddChildNode( unsigned int uChildValue )
{
	WNBitNode* pChildNode = ChildNode(uChildValue);
	if (!pChildNode)
	{
		//要添加的节点不存在
		pChildNode = new WNBitNode;
		//设置节点值
		pChildNode->muNodeValue = uChildValue;
		//添加节点
		AttatchChildNode(pChildNode);
	}
	return pChildNode;
}


void WNBitNode::RemoveChildNode( unsigned int uChildValue )
{
	//记录下分离的子节点
	WNBitNode* pChildNode = DetatchChildNode(uChildValue);
	if (pChildNode)
	{
		//子节点不为空，删除
		delete pChildNode;
	}

}

inline unsigned int WNBitNode::LeafsCount()
{
	return mLeafsInfoMap[muLeafsInfo][ValueCount];
}


inline unsigned int WNBitNode::LeafsAndValueCount()
{
	return mbHasData?(LeafsCount() + 1):LeafsCount();
}


inline void* WNBitNode::Data()
{
	return mbHasData?mppLeafs[mLeafsInfoMap[muLeafsInfo][ValueCount]]:NULL;
}


inline void*& WNBitNode::DataRef()
{
	return (void*&)(mbHasData?mppLeafs[mLeafsInfoMap[muLeafsInfo][ValueCount]]:NullPtr);
}


void* WNBitNode::SetData(void* pNewData, bool bIsWNObject /* = false */)
{
	if (bIsWNObject && pNewData)
	{
		//是WNObject*类型的数据，首先记录新数据
		((WNObject*)pNewData)->Retain();
	}
	void*& pData = DataRef();
	void* pOldData = pData;
	if (mbIsWNObjectPtr && pOldData)
	{
		//当前数据是WNObject*类型的数据，释放引用
		((WNObject*&)pOldData)->Release();
		pData = NULL;
		pOldData = NULL;
	}
	if (mbHasData)
	{
		//已有旧数据
		if (pNewData)
		{
			//新旧数据均存在，使用新数据替换旧数据
			mppLeafs[ValueIndex] = (WNBitNode*)pNewData;
		}
		else
		{
			//旧数据存在且新数据不存在，删除旧数据
			//首先初始化一个新的节点数组ppNewLeafs
			//如果当前子节点数为0，则设置新节点数组为mDefLeafs
			WNBitNode** ppNewLeafs = mDefLeafs;
			if (LeafsCount())
			{
				//如果有子节点
				ppNewLeafs = new WNBitNode*[ArraySizeWithoutValue];
				//复制子节点信息
				memcpy(ppNewLeafs, mppLeafs, LeafsCount() * sizeof(WNBitNode*));
			}
			if (mppLeafs != mDefLeafs)
			{
				//删除老的叶子节点数组
				delete[] mppLeafs;
			}
			//设置ppNewLeafs为当前叶子节点数组
			mppLeafs = ppNewLeafs;
			//设置当前数据为不存在
			mbHasData = false;
		}
	}
	else
	{
		//不存在旧数据
		if (pNewData)
		{
			//新数据存在且旧数据不存在，添加新数据
			//首先初始化一个新的节点数组ppNewLeafs
			WNBitNode** ppNewLeafs = new WNBitNode*[ArraySizeWithValue];
			//设置数据
			ppNewLeafs[ValueIndex] = (WNBitNode*)pNewData;
			if (LeafsCount())
			{
				//存在叶子节点信息
				//拷贝叶子节点信息至新节点数组
				memcpy(ppNewLeafs, mppLeafs, LeafsCount() * sizeof(WNBitNode*));
			}
			if (mppLeafs != mDefLeafs)
			{
				//删除老的叶子节点数组
				delete[] mppLeafs;
			}
			//设置ppNewLeafs为当前叶子节点数组
			mppLeafs = ppNewLeafs;
			//设置当前数据为存在
			mbHasData = true;
		}
		else
		{
			//新旧都数据均不存在
			//不做任何操作
		}
	}
	return pOldData;
}






//===================================================================================================
//WNBitIndex=========================================================================================
//===================================================================================================
WNBitIndex::WNBitIndex()
	:mpNodeHeapManager(NULL)
{
	memset(mpRoot, 0x00,sizeof(mpRoot));
	mpNodeHeapManager = new NodeHeapManager;
}

WNBitIndex::~WNBitIndex()
{
	ClearAll();
	delete mpNodeHeapManager;
}

void* WNBitIndex::SetItem(const void* pKey, unsigned int uKeyLen, void* pData, bool mbIsWNObject /* = false */)
{
	if (!pData)
	{
		//pData为NULL，删除指定的数据
		return RemoveItem(pKey, uKeyLen);
	}
	WNBitNode* pNode = AddNode(pKey, uKeyLen);
	if (!pNode)
	{
		//添加节点失败，返回
		return NULL;
	}
	return pNode->SetData(pData, mbIsWNObject);
}

void* WNBitIndex::GetItem(const void* pKey, unsigned int uKeyLen )
{
	if (!uKeyLen || !pKey)
	{
		//uKeyLen不能为0
		return NULL;
	}
	WNBitNode* pFindNode = FindNode(pKey, uKeyLen);
	return pFindNode?pFindNode->Data():NULL;
}

void* WNBitIndex::RemoveItem(const void* pKey, unsigned int uKeyLen )
{
	if (!uKeyLen || !pKey)
	{
		//uKeyLen不能为0
		return NULL;
	}
	//首先查找第一个字节
	unsigned char* pchKey = (unsigned char*)pKey;
	WNBitNode* pNode = mpRoot[*pchKey];
	if (!pNode)
	{
		//第一个字节不存在,返回NULL
		return NULL;
	}
	else
	{
		//第一个字节存在
		pchKey++;
		uKeyLen--;
	}
	if (!uKeyLen)
	{
		//要移除的的节点关键字只有一个字节
		//第一个字节不做删除操作
		//删除数据后返回
		return pNode->SetData(NULL);
	}
	//关键字长度大于1字节时，遍历关键字所对应的其余所有节点
	//如果关键字存在且某个节点之下都只有一个子节点，且只有关键字所对应的数据节点，则可删除此节点
	
	WNBitNode* pRemoveNode = NULL; //起始无效节点的父节点
	unsigned int uRemoveChildValue = 0; //起始无效节点的子节点值
	WNBitNode* pParentNode = NULL;
	WNBitNode* pFindNode = pNode;
	while (uKeyLen--)
	{
		//查找高字节
		pParentNode = pFindNode;
		pFindNode = pFindNode->ChildNode((*pchKey) >> 4);
		if (!pFindNode)
		{
			return NULL;
		}
		//检查节点是否符合删除条件
		if (!pRemoveNode)
		{
			//没有删除起始节点
			if (pFindNode->LeafsAndValueCount() == 1)
			{
				//当前节点只有一个子节点
				//设置当前节点为删除起始节点
				pRemoveNode = pParentNode;
				uRemoveChildValue = ((*pchKey) >> 4);
			}
		}
		else if (pFindNode->LeafsAndValueCount() != 1)
		{
			//有删除起始节点且当前节点子节点不为1
			//需要重新查找删除起始节点
			pRemoveNode = NULL;
		}
		//查找低字节
		pParentNode = pFindNode;
		pFindNode = pFindNode->ChildNode((*pchKey) & 0x0F);
		if (!pFindNode)
		{
			return NULL;
		}
		//检查节点是否符合删除条件
		if (!pRemoveNode)
		{
			//没有删除起始节点
			if (pFindNode->LeafsAndValueCount() == 1)
			{
				//当前节点只有一个子节点
				//设置当前节点为删除起始节点
				pRemoveNode = pParentNode;
				uRemoveChildValue = ((*pchKey) & 0x0F);
			}
		}
		else if (pFindNode->LeafsAndValueCount() != 1)
		{
			//有删除起始节点且当前节点子节点不为1
			//需要重新查找删除起始节点
			pRemoveNode = NULL;
		}
		pchKey++;
	}
	void* pRet = NULL;
	if (pFindNode)
	{
		//找到关键字所对应的节点
		if (pFindNode->mbHasData)
		{
			//关键字存在
			pRet = pFindNode->SetData(NULL);
		}
		if (pRemoveNode)
		{
			//删除无效节点
			pRemoveNode->RemoveChildNode(uRemoveChildValue);
		}
	}
	return pRet;
}

WNBitNode* WNBitIndex::FindNode(const void* pKey, unsigned int uKeyLen )
{
	if (!uKeyLen || !pKey)
	{
		//uKeyLen不能为0
		return NULL;
	}
	unsigned char* pchKey = (unsigned char*)pKey;
	//寻找第一个字节
	//首先判断第一个字节是否存在
	WNBitNode* pNode = mpRoot[*pchKey];
	if (!pNode)
	{
		//第一个字节不存在
		return NULL;
	}
	else
	{
		pchKey++;
		uKeyLen--;
	}
	WNBitNode* pFindNode = pNode;
	while (uKeyLen--)
	{
		//查找高字节
		pFindNode = pFindNode->ChildNode((*pchKey) >> 4);
		if (!pFindNode)
		{
			return NULL;
		}
		//查找低字节
		pFindNode = pFindNode->ChildNode((*pchKey) & 0x0F);
		if (!pFindNode)
		{
			return NULL;
		}
		pchKey++;
	}
	return pFindNode;
}

WNBitNode* WNBitIndex::AddNode(const void* pKey, unsigned int uKeyLen )
{
	if (!uKeyLen || !pKey)
	{
		//uKeyLen不能为0
		return NULL;
	}
	unsigned char* pchKey = (unsigned char*)pKey;
	//添加第一个字节
	//首先判断第一个字节是否存在
	WNBitNode* pNode = mpRoot[*pchKey];
	if (!pNode)
	{
		//第一个字节不存在
		//创建第一个字节所对应值的节点
		mpRoot[*pchKey] = new WNBitNode;
		pNode = mpRoot[*pchKey];
		pNode->muNodeValue = *pchKey;
		pNode->mbIsEndBit = true;
	}
	pchKey++;
	uKeyLen--;
	WNBitNode* pNewNode = pNode;
	while (uKeyLen--)
	{
		//添加高字节
		pNewNode = pNewNode->AddChildNode((*pchKey) >> 4);
		//添加低字节
		pNewNode = pNewNode->AddChildNode((*pchKey) & 0x0F);
		//设置结束字节
		pNewNode->mbIsEndBit = true;
		pchKey++;
	}
	//返回pKey的关联节点
	return pNewNode;
}

void WNBitIndex::ClearAll()
{
    for (unsigned int i = 0; i < sizeof(mpRoot) / sizeof(WNBitNode*); i++)
	{
		if (mpRoot[i])
		{
			delete mpRoot[i];
			mpRoot[i] = NULL;
		}
	}
}

