/*utf8解析行 Don't remove or modify this first line!*/
#include "WNObject.h"
#include "WNCreticalSection.h"
#include "WNNotificationCenter.h"
#include "WNReadWriteLocker.h"
#include <stdlib.h>
#include <sys/types.h>
#include <sstream>
#include <cstring>
#include "WNWorkThread.h"

using namespace std;
//===================================================================================================
//WNNotification=====================================================================================
//===================================================================================================


WNNotification::WNNotification(unsigned int uParamsCount /* = DEF_NT_PARAMS_COUNT */)
	:muNotificationId(0)
    ,mpNotificationSender(0)
	,mpNotificationData(NULL)
	,mpNotificationCenter(NULL)
	,muParamsCount(uParamsCount)
{
	memset(mParams, 0, muParamsCount * sizeof(unsigned int));
}

WNNotification::~WNNotification()
{
	doRelease(mpNotificationData);
}

const char* WNNotification::NotificationName()
{
	if (mpNotificationCenter)
	{
		return mpNotificationCenter->GetNotificationNameById(muNotificationId);
	}
	else
	{
		return WNNotificationCenter::DefaultCenter()->GetNotificationNameById(muNotificationId);
	}
}

bool WNNotification::IsNotification( const char* lpNotificationName )
{
	if (!muNotificationId)
	{
		return false;
	}
    if (mpNotificationCenter)
	{
		return mpNotificationCenter->GetNotificationIdByName(lpNotificationName) == muNotificationId;
	}
	else
	{
		return WNNotificationCenter::DefaultCenter()->GetNotificationIdByName(lpNotificationName) == muNotificationId;
	}
}


unsigned int WNNotification::GetSize()
{
    return sizeof(WNNotification) + muParamsCount * sizeof(unsigned int) - DEF_NT_PARAMS_COUNT * sizeof(unsigned int);
}

WNNotification* WNNotification::GetCopy()
{
    return NULL;
}

void WNNotification::CopyTo(WNNotification* pNotification)
{

}

//===================================================================================================
//WNNotificationObserver=============================================================================
//===================================================================================================


unsigned int WNNotificationObserver::StartObserveNotification(const char* lpNotificationName, void* pSender /* = NULL */, WNNotificationCenter* pNotificationCenter /* = NULL */)
{
	if (!pNotificationCenter)
	{
		pNotificationCenter = WNNotificationCenter::DefaultCenter();
	}
		
	return pNotificationCenter->AddObserver(this, lpNotificationName, pSender);
}

void WNNotificationObserver::StopObserveNotification(const char* lpNotificationName /* = NULL */, void* pSender /* = NULL */, WNNotificationCenter* pNotificationCenter /* = NULL */)
{
	if (!pNotificationCenter)
	{
		pNotificationCenter = WNNotificationCenter::DefaultCenter();
	}
	return pNotificationCenter->RemoveObserver(this, lpNotificationName, pSender);
}

unsigned int WNNotificationObserver::StartObserveNotificationOnThread(const char *lpNotificationName, WNWorkThread* pThread /*= NULL*/, void *pSender /*= NULL*/, WNNotificationCenter *pNotificationCenter /*= NULL*/)
{
    if (!pThread)
    {
        pThread = WNWorkThread::GetCurrentThread();
    }
    if (!pThread)
    {
        return 0;
    }
    return pThread->AddObserve4Notification(this, lpNotificationName, pSender, pNotificationCenter);
}

void WNNotificationObserver::StopObserveNotificationOnThread(const char* lpNotificationName /*= NULL*/, WNWorkThread* pThread /*= NULL*/, void* pSender /*= NULL*/, WNNotificationCenter* pNotificationCenter /*= NULL*/)
{
    if (!pThread)
    {
        pThread = WNWorkThread::GetCurrentThread();
    }
    if (!pThread)
    {
        return;
    }
    pThread->RemoveObserve4Notification(this, lpNotificationName, pSender, pNotificationCenter);
}



//===================================================================================================
//WNNotificationObserver=============================================================================
//===================================================================================================

WNObserverInfo::WNObserverInfo()
	:mpNotificationReceiver(NULL)
	,mpNotificationSender(NULL)
	,mbIsValid(false)
{

}

//===================================================================================================
//WNNotificationObserverList=========================================================================
//===================================================================================================

WNNotificationObserverList::WNNotificationObserverList()
	:muNotificationId(0)
	,mstrNotificationName("")
    ,mpLocker(NULL)
    ,muNotificationLevel(0)
	,muRemoveCount(0)
{
	mpLocker = new WNCreticalSection;
}


WNNotificationObserverList::~WNNotificationObserverList()
{
	delete mpLocker;
}

void WNNotificationObserverList::AddObserver( WNNotificationObserver* pObserver, void* pSender )
{
	WNNotificationObserverList::PNode pNode = NULL;
	
	mpLocker->Lock();
	//从已有观察者中寻找
	for (pNode = Begin(); pNode != Head(); pNode = pNode->nextNode)
	{
		if (pNode->data.mpNotificationReceiver == pObserver)
		{
			break;
		}
	}
	if (pNode == Head())
	{
		//没有找到
		pNode = new WNNotificationObserverList::Node;
		AppendNode(pNode);
	}
	pNode->data.mpNotificationReceiver = pObserver;
	pNode->data.mpNotificationSender = pSender;
	pNode->data.mbIsValid = true;

	mpLocker->Unlock();
}

void WNNotificationObserverList::RemoveObserver(WNNotificationObserver* pObserver, void* pSender)
{
	WNNotificationObserverList::PNode pNode = NULL;
	mpLocker->Lock();
	//从已有观察者中寻找
	for (pNode = Begin(); pNode != Head(); pNode = pNode->nextNode)
	{
		if ((pNode->data.mpNotificationReceiver == pObserver) && (pNode->data.mpNotificationSender == pSender))
		{
			//判断是否当前对象是否在当前线程的OnReceiveNotification通知中
			if (muNotificationLevel)
			{
				//在通知中
				if (pNode->data.mbIsValid)
				{
					pNode->data.mbIsValid = false;
					muRemoveCount++;
				}
			}
			else
			{
				//不在通知中
				RemoveNode(pNode);
			}
			break;
		}
	}
	mpLocker->Unlock();
}

void WNNotificationObserverList::PostNotification( WNNotification* pNotification, void* pSender /*= NULL*/ )
{
	WNNotificationObserverList::PNode pNode = NULL;
	mpLocker->Lock();
	muNotificationLevel++;
    pNotification->mpNotificationSender = pSender;
	for (pNode = Begin(); pNode != Head(); pNode = pNode->nextNode)
	{
		if ((pNode->data.mpNotificationSender == pSender) && (pNode->data.mbIsValid))
		{
			pNode->data.mpNotificationReceiver->OnReceiveNotification(pNotification);
		}
		else if ((pNode->data.mpNotificationSender == NULL) && (pNode->data.mbIsValid))
		{
			pNode->data.mpNotificationReceiver->OnReceiveNotification(pNotification);
		}
	}
	//删除无效观察者
	if (muRemoveCount && (muNotificationLevel == 1))
	{
		for (pNode = Begin(); pNode != Head();)
		{
			pNode = pNode->nextNode;
			if (!(pNode->prevNode->data.mbIsValid))
			{
				RemoveNode(pNode->prevNode);
			}
		}
		muRemoveCount = 0;
	}
	muNotificationLevel--;
	mpLocker->Unlock();
}


//===================================================================================================
//WNNotificationCenter===============================================================================
//===================================================================================================

#define DefInitNotificationCount 256

WNNotificationCenter* mpDefaultCenter = NULL; //默认的消息中心
//WNNotificationCenter mDefaultCenterCreator; //用于在构造函数中创建默认的消息中心
											//如果存在多个模块，使用第一个作为默认的消息中心

WNNotificationCenter::WNNotificationCenter()
	:mppNotificationObserverList(NULL)
	,muNotificationListMaxCount(0)
	,muNextNotificationId(1)
{

	//有效元素从0开始
	mppNotificationObserverList = new WNNotificationObserverList*[DefInitNotificationCount + 1];
	memset(mppNotificationObserverList, 0x00,sizeof(WNNotificationObserverList*) * (DefInitNotificationCount + 1));
	muNotificationListMaxCount = DefInitNotificationCount;
	mpReadWriteLocker = new WNReadWriteLocker;

}

WNNotificationCenter::~WNNotificationCenter( void )
{
//    printf("enter WNNotificationCenter::~WNNotificationCenter\n");
	mpReadWriteLocker->LockWrite();
	if (mppNotificationObserverList)
	{
		for (unsigned int i = 1; i < muNextNotificationId; i++)
		{
			delete mppNotificationObserverList[i];
		}
		delete[] mppNotificationObserverList;
		mppNotificationObserverList = NULL;
	}
	mpReadWriteLocker->ReleaseWrite();
	delete mpReadWriteLocker;
//    printf("exit WNNotificationCenter::~WNNotificationCenter\n");
}

WNNotificationCenter* WNNotificationCenter::DefaultCenter()
{
	static WNNotificationCenter mDefaultCenter;
	if (!mpDefaultCenter)
	{
		//判断是否已经存在默认的消息中心
		//通过环境变量保证系统中只存在一个消息中心
			 
		char* value=0;
		ostringstream  strKeystream;
		ostringstream  strValuestream;
		strKeystream << "WNNotificationCenter_P" << getpid();
		
		value = getenv(strKeystream.str().c_str());
		if(0 == value){ 
			mpDefaultCenter = &mDefaultCenter;
			strValuestream << (long)mpDefaultCenter;
			if( 0 == setenv(strKeystream.str().c_str(),strValuestream.str().c_str(),1)){
			}
		}else{
			mpDefaultCenter = (WNNotificationCenter*)atol(value);
		}
	}
	return mpDefaultCenter;
}

unsigned int WNNotificationCenter::RegisterNotification( const char* lpNotificationName )
{
	
	unsigned int uId = GetNotificationIdByName(lpNotificationName);

	if (uId)
	{
		return uId;

	}
	//消息未注册
	//注册消息
	mpReadWriteLocker->LockWrite();
	if (muNextNotificationId > muNotificationListMaxCount)
	{
		//消息队列数组已满，需要扩展空间
		//扩展DefInitNotificationCount个大小
		WNNotificationObserverList** pNewArray = new WNNotificationObserverList*[muNotificationListMaxCount + DefInitNotificationCount + 1];
		muNotificationListMaxCount += DefInitNotificationCount;
		//复制原来的指针数组至新数组
		memcpy(pNewArray, mppNotificationObserverList, sizeof(WNNotificationObserverList*) * (muNotificationListMaxCount + 1));
		//删除原来的数组
		delete[] mppNotificationObserverList;
		//指向新数组
		mppNotificationObserverList = pNewArray;
	}
	//加入新id
	mppNotificationObserverList[muNextNotificationId] = new WNNotificationObserverList;
	mppNotificationObserverList[muNextNotificationId]->mstrNotificationName = lpNotificationName;
	mppNotificationObserverList[muNextNotificationId]->muNotificationId = muNextNotificationId;
	uId = muNextNotificationId++;
	mpReadWriteLocker->ReleaseWrite();
	return uId;
}

unsigned int WNNotificationCenter::GetNotificationIdByName( const char* lpNotificationName )
{
	unsigned int uId = 0;
	mpReadWriteLocker->LockRead();

	for (unsigned int i = 1; i < muNextNotificationId; i++)
	{
		if (!::strcmp((const char*)(mppNotificationObserverList[i]->mstrNotificationName.c_str()), lpNotificationName))
		{
			uId = i;
            break;
		}
	}

	mpReadWriteLocker->ReleaseRead();
	return uId;
}

const char* WNNotificationCenter::GetNotificationNameById( unsigned int uNotificationId )
{
	if (!uNotificationId)
	{
		return NULL;
	}
	const char* lpNotificationName = NULL;
	mpReadWriteLocker->LockRead();
	if (uNotificationId < muNextNotificationId)
	{
		lpNotificationName = mppNotificationObserverList[uNotificationId]->mstrNotificationName.c_str();
	}
	mpReadWriteLocker->ReleaseRead();
	return lpNotificationName;
}

unsigned int WNNotificationCenter::AddObserver( WNNotificationObserver* pObserver, const char* lpNotificationName, void* pSender /*= NULL*/ )
{
	if (!pObserver || !lpNotificationName)
	{
		return 0;
	}
	//注册消息
	unsigned int uId = RegisterNotification(lpNotificationName);
	
	//添加观察者
	mppNotificationObserverList[uId]->AddObserver(pObserver, pSender);
	return uId;
}

void WNNotificationCenter::RemoveObserver( WNNotificationObserver* pObserver, const char* lpNotificationName /*= NULL*/, void* pSender /*= NULL*/ )
{
	if (!pObserver)
	{
		return;
	}
	//移除监视
	if (lpNotificationName)
	{
		//在指定消息名称的观察者中移除
		unsigned int uId = GetNotificationIdByName(lpNotificationName);
		if (!uId)
		{
			return;
		}
		mppNotificationObserverList[uId]->RemoveObserver(pObserver, pSender);
	}
	else
	{
		mpReadWriteLocker->LockWrite();
		unsigned int uNextNotificationId = muNextNotificationId;
		mpReadWriteLocker->ReleaseWrite();
		for (unsigned int i = 1; i < uNextNotificationId; i++)
		{
			mppNotificationObserverList[i]->RemoveObserver(pObserver, pSender);
		}
	}
	return;
}

bool WNNotificationCenter::PostNotification( const char* lpNotificationName, WNObject* pNotificationData /*= NULL*/, void* pSender /*= NULL*/ )
{
	if (!lpNotificationName)
	{
		return false;
	}
	unsigned int uId = GetNotificationIdByName(lpNotificationName);
	if (!uId)
	{
		return false;
	}
	return PostNotification(uId, pNotificationData, pSender);
}

bool WNNotificationCenter::PostNotification( unsigned int uNotificationId, WNObject* pNotificationData /*= NULL*/, void* pSender /*= NULL*/ )
{
	mpReadWriteLocker->LockRead();
	WNNotificationObserverList* pList = mppNotificationObserverList[uNotificationId];
	if (!pList)
	{
		mpReadWriteLocker->ReleaseRead();
		return false;
	}
	if (!pList->ListCount())
	{
		mpReadWriteLocker->ReleaseRead();
		return true;
	}

	if (!uNotificationId || (uNotificationId >= muNextNotificationId))
	{
		mpReadWriteLocker->ReleaseRead();
		return false;
	}

	mpReadWriteLocker->ReleaseRead();
	WNNotification* pNotification = WNSafeAlloc(WNNotification);

	pNotification->Retain();
	pNotification->muNotificationId = uNotificationId;
	
	if(pNotificationData)
	   pNotification->mpNotificationData = pNotificationData->Retain();
	
	pNotification->mpNotificationCenter = this;
	pList->PostNotification(pNotification, pSender);
	pNotification->Release();
	return true;
}

bool WNNotificationCenter::PostNotification( WNNotification* pNotification, void* pSender /*= NULL*/ )
{
	mpReadWriteLocker->LockRead();
	unsigned int uNotificationId = pNotification->muNotificationId;
	WNNotificationObserverList* pList = mppNotificationObserverList[uNotificationId];

	if (!pList)
	{
		mpReadWriteLocker->ReleaseRead();
		return false;
	}
	if (!pList->ListCount())
	{
		mpReadWriteLocker->ReleaseRead();
		return true;
	}

	if (!uNotificationId || (uNotificationId >= muNextNotificationId))
	{
		mpReadWriteLocker->ReleaseRead();
		return false;
	}

	mpReadWriteLocker->ReleaseRead();
	pNotification->Retain();

	pNotification->mpNotificationCenter = this;

	pList->PostNotification(pNotification, pSender);

	pNotification->Release();

	return true;
}

