/*utf8解析行 Don't remove or modify this first line!*/
#ifdef __WINOS__
#include "stdafx.h"
#endif
#include "WNObject.h"
#include "WNCreticalSection.h"
#include "stdio.h"

#define InitRefCount 0x3FFFFFFF

WNObject::WNObject()
	:miRetainCount(InitRefCount)
	,mbIsAllocOnHeap(false)
	,miDelayReleaseCount(0)
	,mpDelegate(NULL)
	,mpLoker(NULL)
	,mbIsOwnLocker(false)
    ,mpThis(NULL)
    ,mpTypeFn(NULL)
{

}


WNObject::~WNObject()
{

	ResetSafeLocker();
}


void WNObject::ResetRefInfo()
{
	miRetainCount = InitRefCount;
	miDelayReleaseCount = 0;
}


void WNObject::ResetRefInfoAndDelegate( WNObjectDelegate* pDelegate /*= NULL*/ )
{
	ResetRefInfo();
	mpDelegate = pDelegate;
}


void WNObject::SetSafeLocker( WNCreticalSection* pLocker /*= NULL*/ )
{
	ResetSafeLocker();
	if (pLocker)
	{
		mpLoker = pLocker;
	}
	else
	{
		mpLoker = new WNCreticalSection;
		mbIsOwnLocker = true;
	}
}


WNCreticalSection* WNObject::GetSafeLocker()
{
	return mpLoker;
}


void WNObject::ResetSafeLocker()
{
	if (mpLoker)
	{
		if (mbIsOwnLocker)
		{
			delete mpLoker;
		}
		mpLoker = NULL;
        mbIsOwnLocker = false;
	}
}

WNObject* WNObject::Retain()
{
	WNCreticalSectionAutoLocker locker(mpLoker);
	if (miRetainCount == InitRefCount)
	{
		miRetainCount = 1;
	}
	else
	{
		miRetainCount++;
	}
	return this;
}


WNSmartPtr<WNObject*> WNObject::AutoRelease()
{
	WNCreticalSectionAutoLocker locker(mpLoker);
	return WNSmartPtr<WNObject*>(this->DelayRelease());
}

WNObject* WNObject::Release( bool bClearDelayRelease /*= true*/ )
{
	if (bClearDelayRelease)
	{
        WNCreticalSectionAutoLocker locker(mpLoker);
        int iRetainCount = 0;
        int iDelayReleaseCount = 0;
        int iOldRefCount = 0;
        int iNewRefCount = 0;
        bool bIsChangeZero = false;

        iRetainCount = miRetainCount;
        if (iRetainCount == InitRefCount)
        {
            iRetainCount = 1;
        }
        iDelayReleaseCount = miDelayReleaseCount;
        iOldRefCount = iRetainCount - iDelayReleaseCount;

        if (iOldRefCount < 0)
        {
            printf("error 0 WNObject %p, RefCount is %d", this, iOldRefCount);
            iOldRefCount = 0;
            return NULL;
        }
        else if (!iOldRefCount)
        {
            if (iDelayReleaseCount)
            {
                iNewRefCount = 0;
                bIsChangeZero = true;
                printf("warning 0 WNObject %p, RefCount is %d", this, iOldRefCount);
            }
            else
            {
                printf("error 1 WNObject %p, RefCount is %d", this, iOldRefCount);
                return NULL;
            }
        }
        else
        {
            iNewRefCount = iOldRefCount - 1;
            if (!iNewRefCount)
            {
                bIsChangeZero = true;
            }
        }

        if (!bIsChangeZero)
        {
            miRetainCount = iNewRefCount;
            miDelayReleaseCount = 0;
            return this;
        }
        else
        {
            //need delete or free by mpDelegate
            WNObject* pObject = this;
            if (mpDelegate)
            {
                if (!mpDelegate->OnWNObjectRetainCountIsZero(this))
                {
                    //freed by mpDelegate
                    pObject = NULL;
                }
            }
            if (pObject)
            {
                if (mbIsAllocOnHeap)
                {
                    ResetSafeLocker();
                    locker.mpLocker = NULL;
                    WNObjectTypeFnBase* pTypeFn = this->mpTypeFn;
                    void* pThis = this->mpThis;
                    if (pTypeFn && pThis)
                    {
                        pTypeFn->DoDestroy(pThis);
                    }
                    else
                    {
                        delete this;
                    }
                }
            }
            return NULL;
        }
	}
	else
	{
		WNCreticalSectionAutoLocker locker(mpLoker);
		miRetainCount--;
		return this;
	}
}

WNObject* WNObject::DelayRelease()
{
	WNCreticalSectionAutoLocker locker(mpLoker);
	if (miRetainCount == InitRefCount)
	{
		miRetainCount = 1;
        miDelayReleaseCount = 1;
	}
    else if (miRetainCount)
    {
        miDelayReleaseCount++;
        if (miDelayReleaseCount > miRetainCount)
        {
            miDelayReleaseCount = miRetainCount;
        }
    }
    else
    {
        miRetainCount = 1;
        miDelayReleaseCount = 1;
    }
	return this;
}

WNObject* WNObject::ClearDelayRelease( bool bOlnySetZero /*= FALSE*/ )
{
	WNCreticalSectionAutoLocker locker(mpLoker);
	if (miRetainCount == InitRefCount)
	{
		//初始化状态
		miDelayReleaseCount = 0;
		if (mpDelegate)
		{
			if (!mpDelegate->OnWNObjectRetainCountIsZero(this))
			{
				return this;
			}
		}
		return NULL;
	}

	int iOldRetainCount = miRetainCount;

	if (!bOlnySetZero)
	{
		miRetainCount -= miDelayReleaseCount;
	}
	miDelayReleaseCount = 0;
	if (!miRetainCount && (miRetainCount != iOldRetainCount))
	{
		if (mpDelegate)
		{
			if (!mpDelegate->OnWNObjectRetainCountIsZero(this))
			{
				return this;
			}
		}
		return NULL;
	}
	return this;
}

//void _doRelease( WNObject** ppObj )
//{
//	if (*ppObj)
//	{
//		(*ppObj)->Release();
//		*ppObj = NULL;
//	}
//}
//
//void _doDelayRelease( WNObject** ppObj )
//{
//	if (*ppObj)
//	{
//		(*ppObj)->DelayRelease();
//		*ppObj = NULL;
//	}
//}
