#include <jni.h>
#include <string.h>
#include "BaseSerialPort.h"
#include "TTSManager.h"

//#include <hash_map>
//using namespace std;

#define MOD_LOG_TAG "libgtsdev.so"
#define _GTS_LOG_
#ifdef  _GTS_LOG_
    #include <android/log.h>
    #define LOG (fmt, args...)  __android_log_print(ANDROID_LOG_DEBUG, MOD_LOG_TAG, fmt, ##args)
    #define LOGI(fmt, args...)  __android_log_print(ANDROID_LOG_INFO, MOD_LOG_TAG, fmt, ##args)
    #define LOGE(fmt, args...)  __android_log_print(ANDROID_LOG_ERROR,MOD_LOG_TAG, fmt, ##args)
#else
    #define LOG (...)  
    #define LOGI(...)  
    #define LOGE(...)  
#endif

JNIEnv * g_cbEnv = NULL;
JavaVM * g_jvm = NULL;
jclass g_cbJNIInterface = NULL;
jmethodID g_cbOnRecvCardNumber = NULL;
jmethodID g_cbOnRecvRelayVoltage = NULL;

char* jstringTostring(JNIEnv* env, jstring jstr)
{
	char* rtn = NULL;
	jclass clsstring = env->FindClass("java/lang/String");
	jstring strencode = env->NewStringUTF("utf-8");
	jmethodID mid = env->GetMethodID(clsstring, "getBytes", "(Ljava/lang/String;)[B");
	jbyteArray barr= (jbyteArray)env->CallObjectMethod(jstr, mid, strencode);
	jsize alen = env->GetArrayLength(barr);
	jbyte* ba = env->GetByteArrayElements(barr, JNI_FALSE);
	if (alen > 0)
	{
		rtn = new char[alen + 1];		
		memcpy(rtn, ba, alen);
		rtn[alen] = 0;
	}
	env->ReleaseByteArrayElements(barr, ba, 0);
	return rtn;
}

//注意：当字符串为空时，也会返回一个空字符串  
void split(std::string& s, std::string& delim,std::vector< std::string >* ret)  
{  
	size_t last = 0;  
	size_t index=s.find_first_of(delim,last);  
	while (index!=std::string::npos)  
	{  
		ret->push_back(s.substr(last,index-last));  
		last=index+1;  
		index=s.find_first_of(delim,last);  
	}  
	if (index-last>0)  
	{  
		ret->push_back(s.substr(last,index-last));  
	}  
}  

void * CbOnRecvCardNumber(int type, int num)
{
	if (NULL == g_jvm || NULL == g_cbJNIInterface || NULL == g_cbOnRecvCardNumber)
	{
		return NULL;
	}

	if (NULL == g_cbEnv)
	{	 	
		g_jvm->AttachCurrentThread(&g_cbEnv, NULL);   
		if (NULL == g_cbEnv)
		{	
			return NULL;   
		}
	}
			
	g_cbEnv->CallStaticVoidMethod(g_cbJNIInterface, g_cbOnRecvCardNumber, type, num);
	g_jvm->DetachCurrentThread();
	g_cbEnv = NULL;

	return NULL;
}

void * CbOnRecvRelayVoltage(int part_integer, int part_decimal1, int part_decimal2)
{
	if (NULL == g_jvm || NULL == g_cbJNIInterface || NULL == g_cbOnRecvRelayVoltage)
	{
		return NULL;
	}

	if (NULL == g_cbEnv)
	{	 	
		g_jvm->AttachCurrentThread(&g_cbEnv, NULL);   
		if (NULL == g_cbEnv)
		{	
			return NULL;   
		}
	}
			
	g_cbEnv->CallStaticVoidMethod(g_cbJNIInterface, g_cbOnRecvRelayVoltage, part_integer, part_decimal1, part_decimal2);
	g_jvm->DetachCurrentThread();
	g_cbEnv = NULL;

	return NULL;
}
/*
int GetTimems()
{
  struct timeval tv;
  gettimeofday(&tv, NULL);
  return (int)tv.tv_sec * 1000 + tv.tv_usec/1000;
}*/

class DeviceStateRecver : public WNNotificationObserver
{
public:
    DeviceStateRecver()
	:m_pFarReader(NULL)
	,m_pNearReader(NULL)
	,m_pVoltageReader(NULL){}
    virtual ~DeviceStateRecver(){}

    virtual void OnReceiveNotification(WNNotification* pNotification);
	
	//type为0时表示同时开始远程和近程读卡器，1表示仅开启远程读卡器，2表示仅开启近程读卡器
	void Start(int type)
	{
		StartObserveNotification("GTS_CardNumber");
		StartObserveNotification("GTS_RelayVoltage");
		if(m_pFarReader == NULL && (type == 0 || type == 1))
		{
			m_pFarReader = new BaseSerialPort();
			m_pFarReader->Start(2, 9600, FarReader);
		}
		if(m_pNearReader == NULL && (type == 0 || type == 2))
		{
			m_pNearReader = new BaseSerialPort();
			m_pNearReader->Start(1, 9600, NearReader);
		}
		if(m_pVoltageReader == NULL)
		{
			m_pVoltageReader = new BaseSerialPort();
			m_pVoltageReader->Start(5, 9600, RelayVoltage);
		}
	};
	
	void Stop()
	{
		if(m_pFarReader != NULL)
		{
			m_pFarReader->Stop();
			delete m_pFarReader;
			m_pFarReader = NULL;
		}
		if(m_pNearReader != NULL)
		{
			m_pNearReader->Stop();
			delete m_pNearReader;
			m_pNearReader = NULL;
		}
		if(m_pVoltageReader != NULL)
		{
			m_pVoltageReader->Stop();
			delete m_pVoltageReader;
			m_pVoltageReader = NULL;
		}
		StopObserveNotification("GTS_RelayVoltage");
		StopObserveNotification("GTS_CardNumber");
	};
private:
	BaseSerialPort * m_pFarReader;
	BaseSerialPort * m_pNearReader;
	BaseSerialPort * m_pVoltageReader;//电压读取模块
	
	//hash_map<int, int> m_cardList;
};

void DeviceStateRecver::OnReceiveNotification(WNNotification* pNotification)
{
	if (pNotification->IsNotification("GTS_CardNumber"))
    {
    	CbOnRecvCardNumber(pNotification->mParams[0], pNotification->mParams[1]);
/*		if(m_cardList.find(pNotification->mParams[1]) != m_cardList.end())
		{
			
        }
        else
		{
			m_cardList.find(pNotification->mParams[1]) = GetTimems();
		}
*/
    }
    else if(pNotification->IsNotification("GTS_RelayVoltage"))
    {
	    CbOnRecvRelayVoltage(pNotification->mParams[0], pNotification->mParams[1], pNotification->mParams[2]);
    }
}

DeviceStateRecver * g_pStateRecver = NULL;
TTSManager * g_pTTS = NULL;
bool g_bTTSInit = false;

static jint NativeInit(JNIEnv* env, jclass clazz)
{
	if(g_pTTS == NULL)
	{
		g_pTTS = new TTSManager;
		g_bTTSInit = g_pTTS->InitSoundMap();
		if(!g_bTTSInit)
		{
			LOGE("InitSoundMap failed\n");
			//return 1;
		}
		
	}
	if (NULL == g_cbJNIInterface)
	{
		g_cbJNIInterface = env->FindClass("com/smart/gts/JNIInterface");
		if (NULL == g_cbJNIInterface)
		{
				LOGE("NativeInit, g_cbJNIInterface is null \n");	
				return 1;
		}

		g_cbOnRecvCardNumber = env->GetStaticMethodID(g_cbJNIInterface, "onRecvCardNumber","(II)V");
		if (NULL == g_cbOnRecvCardNumber)
		{
			LOGE("NativeInit, g_cbOnRecvCardNumber is null \n");
			return 1;
		}
		
		g_cbOnRecvRelayVoltage = env->GetStaticMethodID(g_cbJNIInterface, "onRecvRelayVoltage","(III)V");
		if (NULL == g_cbOnRecvRelayVoltage)
		{
			LOGE("NativeInit, g_cbOnRecvRelayVoltage is null \n");
			return 1;
		}

		env->GetJavaVM(&g_jvm);
		if (NULL == g_jvm)
		{
			g_cbJNIInterface = NULL;
			g_cbOnRecvCardNumber = NULL;
			g_cbOnRecvRelayVoltage = NULL;
			LOGE("NativeInit, g_jvm is null\n");
			return 0;
		}
	}

	return 0;
}

static jint NativeUninit(JNIEnv* env, jclass clazz)
{
	g_cbJNIInterface = NULL;
	if(g_pTTS != NULL)
	{
		g_pTTS->UninitSoundMap();
		delete g_pTTS;
		g_pTTS = NULL;
	}
	return 0;
}

static jint Start(JNIEnv* env, jclass clazz, jint type)
{
	if(g_pStateRecver == NULL)
	{
		g_pStateRecver = new DeviceStateRecver();
		g_pStateRecver->Start(type);
	}
		
	return 0;
}

static jint Stop(JNIEnv* env, jclass clazz)
{
	if(g_pStateRecver != NULL)
	{
		g_pStateRecver->Stop();
		delete g_pStateRecver;
		g_pStateRecver = NULL;
	}
	
	return 0;
}

static jint PlayWelcome(JNIEnv* env, jclass clazz)
{
	if(g_pTTS && g_bTTSInit)
	{
		return g_pTTS->PlayWelcome();
	}
	return 0;
}

static jint PlayNoInfoAtStart(JNIEnv* env, jclass clazz)
{
	if(g_pTTS && g_bTTSInit)
	{
		return g_pTTS->PlayNoInfoAtStart();
	}
	return 0;
}

static jint PlayNoInfoAtThisPoint(JNIEnv* env, jclass clazz)
{
	if(g_pTTS && g_bTTSInit)
	{
		return g_pTTS->PlayNoInfoAtThisPoint();
	}
	return 0;
}

static jint PlayRankingInfo(JNIEnv* env, jclass clazz, jint level, jint timeIntPart, jstring timeDecPart)
{
	if(g_pTTS && g_bTTSInit)
	{
		string strDecPart(jstringTostring(env, timeDecPart));
		return g_pTTS->PlayRankingInfo(level, timeIntPart, strDecPart.data());
	}
	return 0;
}

static jint PlayNoUserInfo(JNIEnv* env, jclass clazz)
{
	if(g_pTTS && g_bTTSInit)
	{
		return g_pTTS->PlayNoUserInfo();
	}
	return 0;
}

//为四组，分别表示个人运动信息中的四个可能带小数的数字
static jint PlayUserInfo(JNIEnv* env, jclass clazz, jintArray intPartArr, jstring decPart)
{
	if(g_pTTS && g_bTTSInit)
	{
		jint * iarr = env->GetIntArrayElements(intPartArr, false);  
		if(iarr == NULL) 
		{  
			return 0; /* exception occurred */  
		}  
		int intPart[4];
		for(int i = 0; i < 4;  i++)
		{  
			intPart[i] = iarr[i];  
		}  
		env->ReleaseIntArrayElements(intPartArr, iarr, 0);
		
		string strDecPart(jstringTostring(env, decPart));
		string s = ";";
		std::vector< std::string > strVector;
		split(strDecPart, s, &strVector);
		char * p0 = new char[strVector[0].size() + 1];
		char * p1 = new char[strVector[1].size() + 1];
		char * p2 = new char[strVector[2].size() + 1];
		char * p3 = new char[strVector[3].size() + 1];
		memcpy(p0, strVector[0].data(), strVector[0].size());
		memcpy(p1, strVector[1].data(), strVector[1].size());
		memcpy(p2, strVector[2].data(), strVector[2].size());
		memcpy(p3, strVector[3].data(), strVector[3].size());
		p0[strVector[0].size() + 1] = '\0';
		p1[strVector[1].size() + 1] = '\0';
		p2[strVector[2].size() + 1] = '\0';
		p3[strVector[3].size() + 1] = '\0';
		char * p[] = {p0, p1, p2, p3};
		int ret = g_pTTS->PlayUserInfo(intPart, p);
		delete[] p0;
		delete[] p1;
		delete[] p2;
		delete[] p3;
		
		return ret;
	}
   
	return 0;
}

static jint PlaySetStart(JNIEnv* env, jclass clazz)
{
	if(g_pTTS && g_bTTSInit)
	{
		return g_pTTS->PlaySetStart();
	}
	return 0;
}

static jint PlayUserDistance(JNIEnv* env, jclass clazz, jint intPart, jstring decPart)
{
	if(g_pTTS && g_bTTSInit)
	{
		string strDecPart(jstringTostring(env, decPart));
		return g_pTTS->PlayUserDistance(intPart, strDecPart.data());
	}
	return 0;
}

static jint PlayNoMatchThisCard(JNIEnv* env, jclass clazz)
{
	if(g_pTTS && g_bTTSInit)
	{
		return g_pTTS->PlayNoMatchThisCard();
	}
	return 0;
}

static jint TTSReset(JNIEnv* env, jclass clazz)
{
	if(g_pTTS && g_bTTSInit)
	{
		return g_pTTS->Reset();
	}
	return 0;
}

///////////=======================

static const char* classPathName = "com/smart/gts/JNIInterface";

static JNINativeMethod methods[] = {
  {"NativeInit", "()I", (void*)NativeInit },
  {"NativeUninit", "()I", (void*)NativeUninit },
  {"Start", "(I)I", (void*)Start },
  {"Stop", "()I", (void*)Stop },
  {"PlayWelcome", "()I", (void*)PlayWelcome },
  {"PlayNoInfoAtStart", "()I", (void*)PlayNoInfoAtStart },
  {"PlayNoInfoAtThisPoint", "()I", (void*)PlayNoInfoAtThisPoint },
  {"PlayNoUserInfo", "()I", (void*)PlayNoUserInfo },
  {"PlaySetStart", "()I", (void*)PlaySetStart },
  {"PlayRankingInfo", "(IILjava/lang/String;)I", (void*)PlayRankingInfo},
  {"PlayUserDistance", "(ILjava/lang/String;)I", (void*)PlayUserDistance},
  {"PlayUserInfo", "([ILjava/lang/String;)I", (void*)PlayUserInfo},
  {"PlayNoMatchThisCard", "()I", (void*)PlayNoMatchThisCard },
  {"TTSReset", "()I", (void*)TTSReset }
};

/*
 * Register several native methods for one class.
 */
static int registerNativeMethods(JNIEnv* env, const char* className,
    JNINativeMethod* gMethods, int numMethods)
{
    jclass clazz;

    clazz = env->FindClass(className);
    if (clazz == NULL) {
        ALOGE("Native registration unable to find class '%s'", className);
        return JNI_FALSE;
    }
    if (env->RegisterNatives(clazz, gMethods, numMethods) < 0) {
        ALOGE("RegisterNatives failed for '%s'", className);
        return JNI_FALSE;
    }

    return JNI_TRUE;
}

/*
 * Register native methods for all classes we know about.
 *
 * returns JNI_TRUE on success.
 */
static int registerNatives(JNIEnv* env)
{
  if (!registerNativeMethods(env, classPathName,
                 methods, sizeof(methods) / sizeof(methods[0]))) {
    return JNI_FALSE;
  }

  return JNI_TRUE;
}

/*
 * This is called by the VM when the shared library is first loaded.
 */
 
typedef union {
    JNIEnv* env;
    void* venv;
} UnionJNIEnvToVoid;

jint JNI_OnLoad(JavaVM* vm, void* reserved)
{
    UnionJNIEnvToVoid uenv;
    uenv.venv = NULL;
    jint result = -1;
    JNIEnv* env = NULL;
    
    ALOGI("JNI_OnLoad");

    if (vm->GetEnv(&uenv.venv, JNI_VERSION_1_4) != JNI_OK) {
        ALOGE("ERROR: GetEnv failed");
        goto bail;
    }
    env = uenv.env;

    if (registerNatives(env) != JNI_TRUE) {
        ALOGE("ERROR: registerNatives failed");
        goto bail;
    }
    
    result = JNI_VERSION_1_4;
    
bail:
    return result;
}
