#include "PcmPlayer.h"

#define CHANNEL_MASK_USE_CHANNEL_ORDER 0

PcmPlayer::PcmPlayer()
:mAudioTrack(NULL)
,m_pBufMgr(NULL)
,m_bStart(false)
{

}

PcmPlayer::~PcmPlayer()
{

}

void PcmPlayer::Reset()
{
	if(m_pBufMgr != NULL)
	{
		m_pBufMgr->Reset();
	}
}

bool PcmPlayer::Start()
{
    m_pBufMgr = new WNMemoryPool(1024 * 1024);//(64 * 1024);

	int32_t numChannels = 1, channelMask = 0;
	// playing to an AudioTrack, set up mask if necessary
    audio_channel_mask_t audioMask = channelMask == CHANNEL_MASK_USE_CHANNEL_ORDER ?
            audio_channel_out_mask_from_count(numChannels) : channelMask;
    if (0 == audioMask) 
    {
        return false;
    }

    mAudioTrack = new AudioTrack(
            AUDIO_STREAM_MUSIC, 16000, AUDIO_FORMAT_PCM_16_BIT, audioMask,
            0, AUDIO_OUTPUT_FLAG_NONE, &AudioCallback, this, 0);


    //mAudioTrack = new AudioTrack(
    //        AUDIO_STREAM_MUSIC, 8000, AUDIO_FORMAT_PCM_16_BIT, audioMask,
    //        0, AUDIO_OUTPUT_FLAG_NONE, &AudioCallback, this, 0);

    status_t err;
    if ((err = mAudioTrack->initCheck()) != OK)
    {
        mAudioTrack.clear();

        return false;
    }

    int64_t mLatencyUs = (int64_t)mAudioTrack->latency() * 1000;
    size_t mFrameSize = mAudioTrack->frameSize();

    mAudioTrack->start();

    m_bStart = true;

    return true;
}

void PcmPlayer::Stop()
{
    printf("enter PcmPlayer::stop\n");
    m_bStart = false;
	if(mAudioTrack != NULL)
	{
		mAudioTrack->stop();
		mAudioTrack = NULL;
	}

    if (m_pBufMgr != NULL)
    {
        delete m_pBufMgr;
        m_pBufMgr = NULL;
    }
    printf("exit PcmPlayer::stop\n");
}

// static
void PcmPlayer::AudioCallback(int event, void *user, void *info) {
    static_cast<PcmPlayer *>(user)->AudioCallback(event, info);
}

void PcmPlayer::AudioCallback(int event, void *info) {
    switch (event) {
    case AudioTrack::EVENT_MORE_DATA:
        {
        AudioTrack::Buffer *buffer = (AudioTrack::Buffer *)info;
        size_t numBytesWritten = FillBuffer(buffer->raw, buffer->size);
        buffer->size = numBytesWritten;
        }
        break;

    case AudioTrack::EVENT_STREAM_END:
        //mReachedEOS = true;
        //notifyAudioEOS();
    	printf("ReachedEOS...\n");
        break;
    }
}

bool PcmPlayer::AddData(unsigned char * data, int len)
{
	if(data != NULL && m_pBufMgr != NULL)
	{
		m_pBufMgr->Write(data, len);
	}

	return true;
}

size_t PcmPlayer::FillBuffer(void *data, size_t size)
{
    return m_pBufMgr->Read((unsigned char*)data, size);
}