#include "TTSManager.h"

const int DIAN = 11;
const int MI = 12;
//const int KMH = 13;
const int WELCOME = 14;
const int JJPM1 = 15;//您在当前中继点的排名为第
const int JJPM2 = 16;//名，距离第1名
const int JJPM3 = 17;//分钟
const int GRXX1 = 18;//您当前已登山里程
const int GRXX2 = 19;//米；登山时间
const int GRXX3 = 20;//小时，剩余里程
const int GRXX4 = 21;//米，平均速度
const int GRXX5 = 22;//米每小时
const int NOINFO_START = 23;//当前中继点为赛事起点，暂时没有排名信息
const int NOINFO_CUR = 24;//您在当前点暂时没有排名信息
const int NOUSERINFO = 25;//获取不到您的运动信息
const int SET_START = 26;//您已设置当前中继点为登山起点
const int DISTANCE = 27;//您的登山里程为
const int NO_MATCH = 28;//该卡当前无赛事

TTSManager::TTSManager()
:m_pNumBuf(NULL)
{
	
}

TTSManager::~TTSManager()
{
	
}

bool TTSManager::ReadPcmFile(int num)
{
	char szFile[128];
	FILE * fp = NULL;
	memset(szFile, 0, 128);
	sprintf(szFile, "/mnt/sdcard/GTSFiles/Sound/%d.pcm", num);
	fp = fopen(szFile, "rb");
	if(fp == NULL)
	{
		LOGE("open %s failed\n", szFile);
		return false;
	}
	
	NumData * pNum = new NumData;
	fseek(fp, 0, SEEK_END);
	pNum->iLen = ftell(fp);
	fseek(fp, 0, SEEK_SET);
	pNum->pData = new char[pNum->iLen];
	fread(pNum->pData, 1, pNum->iLen, fp);
	fclose(fp);
	
	m_numMap.insert(pair<int, NumData*>(num, pNum));
	
	return true;
}

bool TTSManager::InitSoundMap()
{
	m_pNumBuf = new char[1024 * 1024];
	
	for(int i = 0; i < 29; i++)
	{
		if(!ReadPcmFile(i))
		{
			return false;
		}
	}
	
	if(!ReadPcmFile(10000)) return false;
	if(!ReadPcmFile(1000)) return false;
	if(!ReadPcmFile(100)) return false;
	if(!ReadPcmFile(10)) return false;
	
	m_player.Start();
	
	return true;
}

void TTSManager::UninitSoundMap()
{
	NumData * pNum = NULL;
	hash_map<int, NumData*>::iterator it = m_numMap.begin();
	while(it != m_numMap.end())//遍历
	{
		pNum = it++->second;
		if(pNum != NULL)
		{
			if(pNum->pData != NULL)
			{
				delete[] pNum->pData;
				pNum->pData = NULL;
			}
			delete pNum;
		}
	}
	m_numMap.clear();
	
	if(m_pNumBuf != NULL)
	{
		delete[] m_pNumBuf;
		m_pNumBuf = NULL;
	}
	
	m_player.Stop();
}

int TTSManager::CheckNumber(int& num, int mul, int pre/* = 0*/)
{
	int ret = -1;
	if(num >= mul)
	{
		ret = num / mul;
		if(ret > 0)
		{
			num -= mul * ret;
		}
	}
	
	if(ret < 0 && pre > 0)
	{
		ret = 0;
	}
	
	return ret;
}

int TTSManager::CopyPcmBuf(int num, char * pBuf)
{
	int iSpaceLen = 4000;
	NumData * pNum = NULL;
	hash_map<int, NumData*>::iterator it = m_numMap.find(num);
	if(it != m_numMap.end())
	{
		int len = 0;
		pNum = it->second;
		memcpy(pBuf + len, pNum->pData, pNum->iLen);
		len += pNum->iLen;
		memset(pBuf + len, 0, iSpaceLen);
		len += iSpaceLen;
		
		return len;
	}
	return 0;
}
	
int TTSManager::CreateNumberData(int intPart, const char* decPart, char * pBuf)
{
	LOGE("CreateNumberData, intPart = %d, decPart = %s", intPart, decPart);
	int iPos = 0;
	int iTemp = 0;
	if(intPart < 0)
	{
		return 0;
	}
	else if(intPart > 0)
	{
		int temp = intPart;
		int w = CheckNumber(temp, 10000);
		int q = CheckNumber(temp, 1000, w);
		int b = CheckNumber(temp, 100, q);
		int s = CheckNumber(temp, 10, b);
		int g = CheckNumber(temp, 1, s);
		
		//LOGE("%d, %d, %d, %d, %d\n", w, q, b, s, g);
		
		if(w > 0)
		{	
			iTemp = CopyPcmBuf(w, pBuf + iPos);
			if(iTemp <= 0)
			{
				return 0;
			}
			iPos += iTemp;
			
			iTemp = CopyPcmBuf(10000, pBuf + iPos);
			if(iTemp <= 0)
			{
				return 0;
			}
			iPos += iTemp;
		}
		
		if((w > 0 && q >= 0) || (w <= 0 && q > 0))
		{
			iTemp = CopyPcmBuf(q, pBuf + iPos);
			if(iTemp <= 0)
			{
				return 0;
			}
			iPos += iTemp;
			
			if(q > 0)
			{
				iTemp = CopyPcmBuf(1000, pBuf + iPos);
				if(iTemp <= 0)
				{
					return 0;
				}
				iPos += iTemp;
			}
		}
		
		if((q > 0 && b >= 0) || (q <= 0 && b > 0))
		{
			iTemp = CopyPcmBuf(b, pBuf + iPos);
			if(iTemp <= 0)
			{
				return 0;
			}
			iPos += iTemp;
			
			if(b > 0)
			{
				iTemp = CopyPcmBuf(100, pBuf + iPos);
				if(iTemp <= 0)
				{
					return 0;
				}
				iPos += iTemp;
			}
		}
		
		if((b > 0 && s >= 0) || (b <= 0 && s > 0))
		{
			iTemp = CopyPcmBuf(s, pBuf + iPos);
			if(iTemp <= 0)
			{
				return 0;
			}
			iPos += iTemp;
			
			if(s > 0)
			{
				iTemp = CopyPcmBuf(10, pBuf + iPos);
				if(iTemp <= 0)
				{
					return 0;
				}
				iPos += iTemp;
			}
		}
		
		if((s > 0 && g >= 0) || (s <= 0 && g > 0))
		{
			iTemp = CopyPcmBuf(g, pBuf + iPos);
			if(iTemp <= 0)
			{
				return 0;
			}
			iPos += iTemp;
		}
	}
	else if(intPart == 0)
	{
		iTemp = CopyPcmBuf(intPart, pBuf + iPos);
		if(iTemp <= 0)
		{
			return 0;
		}
		iPos += iTemp;
	}
	
	iTemp = atoi(decPart);
	if(iTemp > 0)
	{
		iTemp = CopyPcmBuf(DIAN, pBuf + iPos);
		if(iTemp <= 0)
		{
			return 0;
		}
		iPos += iTemp;
		
		int x = -1;
		char c;
		int counts = strlen(decPart); 
		for(int i = 0; i < counts; i++)
		{
			c = decPart[i];
			x = atoi(&c);
			iTemp = CopyPcmBuf(x, pBuf + iPos);
			if(iTemp <= 0)
			{
				return 0;
			}
			iPos += iTemp;
		}
	}
	
	return iPos;
}

int TTSManager::PlayWelcome()
{
	return PlaySoundById(WELCOME);
}

int TTSManager::PlayRankingInfo(int level, int timeIntPart, const char * timeDecPart)
{
	int iLen = 0;
	int iTemp = 0;
	
	iTemp = CopyPcmBuf(JJPM1, m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;

	iTemp = CreateNumberData(level, "0", m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;
	
	memset(m_pNumBuf + iLen, 0, 8000);
	iLen += 8000;
	
	iTemp = CopyPcmBuf(JJPM2, m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;
	
	iTemp = CreateNumberData(timeIntPart, timeDecPart, m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;
	
	iTemp = CopyPcmBuf(JJPM3, m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;

	m_player.Reset();
	m_player.AddData((unsigned char*)m_pNumBuf, iLen);
	
	return 1;
}

int TTSManager::PlaySoundById(int soundid)
{
	int iLen = 0;
	int iTemp = 0;
	
	iTemp = CopyPcmBuf(soundid, m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;
	
	m_player.Reset();
	m_player.AddData((unsigned char*)m_pNumBuf, iLen);
	
	return 1;
}

int TTSManager::PlayNoInfoAtStart()
{
	return PlaySoundById(NOINFO_START);
}

int TTSManager::PlayNoInfoAtThisPoint()
{
	return PlaySoundById(NOINFO_CUR);
}

int TTSManager::PlayNoUserInfo()
{
	return PlaySoundById(NOUSERINFO);
}

//参数为四组，分别表示个人运动信息中的四个可能带小数的数字
int TTSManager::PlayUserInfo(int * intPart, char * decPart[])
{
	int iLen = 0;
	int iTemp = 0;
	
	iTemp = CopyPcmBuf(GRXX1, m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;

	iTemp = CreateNumberData(intPart[0], decPart[0], m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;
	
	//memset(m_pNumBuf + iLen, 0, 8000);
	//iLen += 8000;
	
	iTemp = CopyPcmBuf(GRXX2, m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;
	
	iTemp = CreateNumberData(intPart[1], decPart[1], m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;
	
	iTemp = CopyPcmBuf(GRXX3, m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;
	
	iTemp = CreateNumberData(intPart[2], decPart[2], m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;
	
	iTemp = CopyPcmBuf(GRXX4, m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;
	
	iTemp = CreateNumberData(intPart[3], decPart[3], m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;
	
	iTemp = CopyPcmBuf(GRXX5, m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;
	
	m_player.Reset();
	m_player.AddData((unsigned char*)m_pNumBuf, iLen);
	
	return 1;
}

int TTSManager::PlaySetStart()
{
	return PlaySoundById(SET_START);
}

int TTSManager::PlayUserDistance(int intPart, const char * decPart)
{
	int iLen = 0;
	int iTemp = 0;
	
	iTemp = CopyPcmBuf(DISTANCE, m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;

	iTemp = CreateNumberData(intPart, decPart, m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;
	
	memset(m_pNumBuf + iLen, 0, 8000);
	iLen += 8000;
	
	iTemp = CopyPcmBuf(MI, m_pNumBuf + iLen);
	if(iTemp <= 0)
	{
		return 0;
	}
	iLen += iTemp;
	
	m_player.Reset();
	m_player.AddData((unsigned char*)m_pNumBuf, iLen);
	
	return 1;
}

int TTSManager::PlayNoMatchThisCard()
{
	return PlaySoundById(NO_MATCH);
}

int TTSManager::Reset()
{
	m_player.Reset();
	return 1;
}



